<?php

defined('_JEXEC') or die('Direct Access to ' . basename(__FILE__) . ' is not allowed.');

/**
 *
 * @author jasmin.s@leftor.ba
 * @version $Id: eps.php$
 * @package VirtueMart
 * @subpackage payment
 * @copyright Copyright (C) 2017 Leftor d.o.o. - All rights reserved.
 * @license http://www.gnu.org/copyleft/gpl.html GNU/GPL, see LICENSE.php
 * VirtueMart is free software. This version may have been modified pursuant
 * to the GNU General Public License, and as distributed it includes or
 * is derivative of works licensed under the GNU General Public License or
 * other free or open source software licenses.
 * See /administrator/components/com_virtuemart/COPYRIGHT.php for copyright notices and details.
 *
 */
require (VMPATH_ROOT . DS . 'plugins' . DS . 'vmpayment' . DS .'eps'. DS . 'vendor' . DS . 'autoload.php');

if (!class_exists('vmPSPlugin'))
    require(JPATH_VM_PLUGINS . DS . 'vmpsplugin.php');

use at\externet\eps_bank_transfer;

class plgVmPaymentEps extends vmPSPlugin {

    public static $_this = false;

    function __construct(& $subject, $config) {

        parent::__construct($subject, $config);

        $this->_loggable = true;
        $this->tableFields = array_keys($this->getTableSQLFields());
        $this->_tablepkey = 'id';
        $this->_tableId = 'id';

        $varsToPush = array(
            'eps_user_id' => array('', 'char'),
            'eps_pin' => array('', 'char'),
            'eps_bic' => array('', 'char'),
            'eps_iban' => array('', 'char'),
            'payment_currency' => array('', 'int'),
            'payment_logos' => array('', 'char'),
            'test_mode' => array(0, 'char'),
            'debug' => array(0, 'int'),
            'status_pending' => array('', 'char'),
            'status_success' => array('', 'char'),
            'status_canceled' => array('', 'char'),
            'countries' => array(0, 'char')
        );

        $this->setConfigParameterable($this->_configTableFieldName, $varsToPush);

    }

    function _getEpsDetails($method)
    {
        $epsDetails = array(
            'user_id' => $method->eps_user_id,
            'pin' => $method->eps_pin,
            'bic' => $method->eps_bic,
            'iban' => $method->eps_iban,
            'env' => $method->test_mode
                ? 'https://routing.eps.or.at/appl/epsSO-test/transinit/eps/v2_5'
                : 'https://routing.eps.or.at/appl/epsSO/transinit/eps/v2_5'
        );

        return $epsDetails;
    }

    public function getVmPluginCreateTableSQL() {

        return $this->createTableSQL('Payment Eps Table');
    }

    function getTableSQLFields() {

        $SQLfields = array(
            'id' => 'int(11) unsigned NOT NULL AUTO_INCREMENT ',
            'virtuemart_order_id' => 'int(11) UNSIGNED DEFAULT NULL',
            'order_number' => 'char(32) DEFAULT NULL',
            'virtuemart_paymentmethod_id' => 'mediumint(1) UNSIGNED DEFAULT NULL',
            'payment_name' => 'char(255) NOT NULL DEFAULT \'\' ',
            'payment_order_total' => 'decimal(15,5) NOT NULL DEFAULT \'0.00000\' ',
            'payment_currency' => 'char(3) ',
            'tax_id' => 'smallint(1) DEFAULT NULL',
            'eps_response' => 'varchar(255)  ',
            'eps_response_order_number' => 'char(20) DEFAULT NULL'
        );
        return $SQLfields;
    }

    function plgVmConfirmedOrder($cart, $order) {

        if (!($method = $this->getVmPluginMethod($order['details']['BT']->virtuemart_paymentmethod_id))) {
            return null; // Another method was selected, do nothing
        }
        if (!$this->selectedThisElement($method->payment_element)) {
            return false;
        }
        $session = JFactory::getSession();
        $return_context = $session->getId();
        $this->_debug = $method->debug;

        if (!class_exists('VirtueMartModelCurrency'))
            require(VMPATH_ADMIN . DS . 'models' . DS . 'currency.php');

        $new_status = 'P';

        $usrBT = $order['details']['BT'];
        $address = ((isset($order['details']['BT'])) ? $order['details']['BT'] : $order['details']['ST']);

        if (!class_exists('TableVendors'))
            require(VMPATH_ADMIN . DS . 'table' . DS . 'vendors.php');
        $vendorModel = VmModel::getModel('Vendor');
        $vendorModel->setId(1);
        $vendor = $vendorModel->getVendor();
        $vendorModel->addImages($vendor, 1);
        $this->getPaymentCurrency($method);
        $q = 'SELECT `currency_code_3` FROM `#__virtuemart_currencies` WHERE `virtuemart_currency_id`="' . $method->payment_currency . '" ';
        $db = JFactory::getDBO();
        $db->setQuery($q);
        $currency_code_3 = $db->loadResult();

        $paymentCurrency = CurrencyDisplay::getInstance($method->payment_currency);
        $totalInPaymentCurrency = round($paymentCurrency->convertCurrencyTo($method->payment_currency, $order['details']['BT']->order_total, false), 2);
        $cd = CurrencyDisplay::getInstance($cart->pricesCurrency);

        $epsDetails = $this->_getEpsDetails($method);

        if (empty($epsDetails['user_id'])) {
            vmInfo(vmText::_('VMPAYMENT_EPS_SELLER_ID_NOT_SET'));
            return false;
        }

        // Prepare data that should be stored in the database
        $dbValues['order_number'] = $order['details']['BT']->order_number;
        $dbValues['payment_name'] = $this->renderPluginName($method, $order);
        $dbValues['virtuemart_paymentmethod_id'] = $cart->virtuemart_paymentmethod_id;
        $dbValues['eps_custom'] = $return_context;
        $dbValues['payment_currency'] = $method->payment_currency;
        $dbValues['payment_order_total'] = $totalInPaymentCurrency;
        $dbValues['tax_id'] = $method->tax_id;
        $this->storePSPluginInternalData($dbValues);

        // Connection credentials
        $userID = $epsDetails['user_id'];
        $pin    = $epsDetails['pin'];
        $bic    = $epsDetails['bic'];
        $iban   = $epsDetails['iban'];
        $fullName = $address->first_name . ' ' . $address->last_name;
        $orderTotal = $totalInPaymentCurrency*100;

        $targetUrl = $epsDetails['env'];
        
        $responseUrl = JRoute::_(JURI::root() .'plugins/vmpayment/eps/confirm.php?pm='.$order['details']['BT']->virtuemart_paymentmethod_id );
        //$successUrl = JRoute::_(JURI::root()."index.php?option=com_virtuemart&amp;view=orders&amp;layout=details&amp;order_number=".$order['details']['BT']->order_number."&amp;order_pass=".$order['details']['BT']->order_pass, false);
        $failUrl = JRoute::_(JURI::root() . 'index.php?option=com_virtuemart&amp;view=pluginresponse&amp;task=pluginUserPaymentCancel');
        $successUrl = JRoute::_(JURI::root() . 'index.php?option=com_virtuemart&amp;view=pluginresponse&amp;task=pluginNotification');

        $transferMsgDetails = new eps_bank_transfer\TransferMsgDetails(
            $responseUrl,
            $successUrl,
            $failUrl
        );

        $transferInitiatorDetails = new eps_bank_transfer\TransferInitiatorDetails(
            $userID,
            $pin,
            $bic,
            $fullName,
            $iban,
            $cart->id,
            intval($orderTotal),
            $transferMsgDetails
        );
        $transferInitiatorDetails->RemittanceIdentifier = $order['details']['BT']->order_number;
        $transferInitiatorDetails->SetExpirationMinutes(60);

        foreach ($cart->products as $key => $product) {
            $article = new eps_bank_transfer\WebshopArticle(
                substr(strip_tags($product->product_name), 0, 127),
                $product->quantity,
                intval($cart->pricesUnformatted[$key]['salesPrice']*100)
            );
            $transferInitiatorDetails->WebshopArticles[] = $article;
        }

        // Send TransferInitiatorDetails to Scheme Operator
        $soCommunicator = new eps_bank_transfer\SoCommunicator();

        $plain = $soCommunicator->SendTransferInitiatorDetails($transferInitiatorDetails, $targetUrl);
        $xml = new \SimpleXMLElement($plain);
        $soAnswer = $xml->children(eps_bank_transfer\XMLNS_epsp);
        $errorDetails = $soAnswer->BankResponseDetails->ErrorDetails;

        if (('' . $errorDetails->ErrorCode) != '000')
        {
            $errorCode = '' . $errorDetails->ErrorCode;
            $errorMsg = '' . $errorDetails->ErrorMsg;

            print 'Error code: ' . $errorCode . ', error message: ' . $errorMsg;
        }
        else
        {
            $redirectUrl = $soAnswer->BankResponseDetails->ClientRedirectUrl->__toString();
            $modelOrder = VmModel::getModel ('orders');
			$order['order_status'] = $new_status;
			$order['customer_notified'] = 0;
			$order['comments'] = 'Redirected to eps system, waiting for payment';
			$modelOrder->updateStatusForOneOrder ($order['details']['BT']->virtuemart_order_id, $order, TRUE);
			//$cart->emptyCart();
            header('Location: ' . $redirectUrl);
        }
    }

    function plgVmgetPaymentCurrency($virtuemart_paymentmethod_id, &$paymentCurrencyId) {

        if (!($method = $this->getVmPluginMethod($virtuemart_paymentmethod_id))) {
            return null; // Another method was selected, do nothing
        }
        if (!$this->selectedThisElement($method->payment_element)) {
            return false;
        }
        $this->getPaymentCurrency($method);
        $paymentCurrencyId = $method->payment_currency;
    }

    function plgVmOnPaymentNotification()
    {
    	$cart =  VirtueMartCart::getCart();
    	$cart->emptyCart();
        $msg = 'Thank you for your order!';
        $app = JFactory::getApplication();
        $app->redirect(JRoute::_('index.php?option=com_virtuemart&view=order&layout=details', false), $msg);
    }

    function plgVmOnUserPaymentCancel()
    {
        $errorCode = vRequest::get('epserrorcode');
        $errorMsg = $this->getErrorMsg($errorCode);
        vmError($errorMsg);
        return;
    }

    private function getErrorMsg($errorCode)
    {
        $errors = array(
            'ERROR1' => vmText::sprintf('VMPAYMENT_EPS_ERROR1'),
            'ERROR2' => vmText::sprintf('VMPAYMENT_EPS_ERROR2'),
            'ERROR3' => vmText::sprintf('VMPAYMENT_EPS_ERROR3')
        );
        return $errors[$errorCode];
    }

    function paymentCancel($order_id) {

        if (!class_exists('VirtueMartModelOrders'))
            require( JPATH_VM_ADMINISTRATOR . DS . 'models' . DS . 'orders.php' );

        $order_number = $order_id;
        if (!$order_number)
            return false;
        $db = JFactory::getDBO();
        $query = 'SELECT ' . $this->_tablename . '.`virtuemart_order_id` FROM ' . $this->_tablename . " WHERE  `order_number`= '" . $order_number . "'";

        $db->setQuery($query);
        $virtuemart_order_id = $db->loadResult();

        if (!$virtuemart_order_id) {
            return null;
        }
        $this->handlePaymentUserCancel($virtuemart_order_id);

        return true;
    }

    function plgVmOnShowOrderBEPayment($virtuemart_order_id, $payment_method_id) {

        if (!$this->selectedThisByMethodId($payment_method_id)) {
            return null; // Another method was selected, do nothing
        }

        if (!($paymentTable = $this->getDataByOrderId ($virtuemart_order_id))) {
            return NULL;
        }
        vmLanguage::loadJLang('com_virtuemart');
        $html = '<table class="adminlist table">' . "\n";
        $html .= $this->getHtmlHeaderBE ();
        $html .= $this->getHtmlRowBE ('COM_VIRTUEMART_PAYMENT_NAME', $paymentTable->payment_name);
        $html .= $this->getHtmlRowBE ('EPS_ORDER_NUMBER', $paymentTable->eps_response_order_number);
        $html .= '</table>' . "\n";
        return $html;

        return $html;
    }

    function _getPaymentResponseHtml($epsTable, $payment_name) {

        $html = '<table>' . "\n";
        $html .= $this->getHtmlRow('EPS_PAYMENT_NAME', $payment_name);
        if (!empty($epsTable)) {
            $html .= $this->getHtmlRow('EPS_ORDER_NUMBER', $epsTable['order_number']);
            $html .= $this->getHtmlRow('EPS_AMOUNT', $epsTable['total'] . " " . $epsTable['currency_code']);
        }
        $html .= '</table>' . "\n";

        return $html;
    }

    function getCosts(VirtueMartCart $cart, $method, $cart_prices) {
        if (preg_match('/%$/', $method->cost_percent_total)) {
            $cost_percent_total = substr($method->cost_percent_total, 0, -1);
        } else {
            $cost_percent_total = $method->cost_percent_total;
        }
        return ($method->cost_per_transaction + ($cart_prices['salesPrice'] * $cost_percent_total * 0.01));
    }


    protected function checkConditions($cart, $method, $cart_prices) {


        $address = (($cart->ST == 0) ? $cart->BT : $cart->ST);

        $amount = $cart_prices['salesPrice'];
        $amount_cond = ($amount >= $method->min_amount AND $amount <= $method->max_amount
            OR
            ($method->min_amount <= $amount AND ($method->max_amount == 0) ));

        $countries = array();
        if (!empty($method->countries)) {
            if (!is_array($method->countries)) {
                $countries[0] = $method->countries;
            } else {
                $countries = $method->countries;
            }
        }

        if (!is_array($address)) {
            $address = array();
            $address['virtuemart_country_id'] = 0;
        }

        if (!isset($address['virtuemart_country_id']))
            $address['virtuemart_country_id'] = 0;
        if (in_array($address['virtuemart_country_id'], $countries) || count($countries) == 0) {
            if ($amount_cond) {
                return true;
            }
        }

        return false;
    }

    function plgVmOnStoreInstallPaymentPluginTable($jplugin_id) {

        return $this->onStoreInstallPluginTable($jplugin_id);
    }

    public function plgVmOnSelectCheckPayment(VirtueMartCart $cart) {
        return $this->OnSelectCheck($cart);
    }

    public function plgVmDisplayListFEPayment(VirtueMartCart $cart, $selected = 0, &$htmlIn) {
        return $this->displayListFE($cart, $selected, $htmlIn);
    }

    public function plgVmonSelectedCalculatePricePayment(VirtueMartCart $cart, array &$cart_prices, &$cart_prices_name) {
        return $this->onSelectedCalculatePrice($cart, $cart_prices, $cart_prices_name);
    }

    function plgVmOnCheckAutomaticSelectedPayment(VirtueMartCart $cart, array $cart_prices = array()) {
        return $this->onCheckAutomaticSelected($cart, $cart_prices);
    }

    public function plgVmOnShowOrderFEPayment($virtuemart_order_id, $virtuemart_paymentmethod_id, &$payment_name) {
        $this->onShowOrderFE($virtuemart_order_id, $virtuemart_paymentmethod_id, $payment_name);
    }

    function plgVmonShowOrderPrintPayment($order_number, $method_id) {
        return $this->onShowOrderPrint($order_number, $method_id);
    }

    function plgVmDeclarePluginParamsPayment($name, $id, &$data) {
        return $this->declarePluginParams('payment', $name, $id, $data);
    }

    function plgVmDeclarePluginParamsPaymentVM3( &$data) {
        return $this->declarePluginParams('payment', $data);
    }

    function plgVmSetOnTablePluginParamsPayment($name, $id, &$table) {
        return $this->setOnTablePluginParams($name, $id, $table);
    }

}

// No closing tag
